**目录：**

<!-- TOC -->

- [Java](#java)
    - [基础](#基础)
    - [并发](#并发)
    - [JVM](#jvm)
    - [Java8 新特性](#java8-新特性)
    - [代码优化](#代码优化)
    - [面试](#面试)
- [网络](#网络)
- [操作系统](#操作系统)
- [数据结构](#数据结构)
- [算法](#算法)
    - [入门](#入门)
    - [经典](#经典)
    - [面试](#面试-1)
- [数据库](#数据库)
- [系统设计](#系统设计)
    - [设计模式](#设计模式)
    - [常用框架](#常用框架)
        - [Spring/SpringBoot](#springspringboot)
        - [Netty](#netty)
    - [分布式](#分布式)
    - [网站架构](#网站架构)
    - [底层](#底层)
- [软件设计之道](#软件设计之道)
- [其他](#其他)

<!-- /TOC -->

## Java

### 基础

- **[《Head First Java》](https://book.douban.com/subject/2000732/)** ： 可以说是我的 Java 启蒙书籍了，我个人觉得还是很适合稍微有一点点经验的新手来阅读的当然也适合我们用来温故 Java 知识点。*ps:刚入门编程，最好的方式还是通过看视频来学习。*
- **[《Java 核心技术卷 1+卷 2》](https://book.douban.com/subject/25762168/)**: 很棒的两本书，建议有点 Java 基础之后再读，介绍的还是比较深入的，非常推荐。这两本书我一般也会用来巩固知识点或者当做工具书参考，是两本适合放在自己身边的好书。
- **[《Java 编程思想 (第 4 版)》](https://book.douban.com/subject/2130190/)**（推荐，豆瓣评分 9.1，3.2K+人评价）：大部分人称之为Java领域的圣经，但我不推荐初学者阅读，有点劝退的味道。稍微有点基础后阅读更好。
- **[《JAVA 网络编程 第 4 版》](https://book.douban.com/subject/26259017/)**：  可以系统的学习一下网络的一些概念以及网络编程在 Java 中的使用。
- **[《Java性能权威指南》](https://book.douban.com/subject/26740520/)**：O'Reilly 家族书，性能调优的入门书，我个人觉得性能调优是每个 Java 从业者必备知识，这本书的缺点就是太老了，但是这本书可以作为一个实战书，尤其是 JVM 调优！不适合初学者。前置书籍：《深入理解 Java 虚拟机》

### 并发

- **[《Java 并发编程之美》](<https://book.douban.com/subject/30351286/>)** ：**我觉得这本书还是非常适合我们用来学习 Java 多线程的。这本书的讲解非常通俗易懂，作者从并发编程基础到实战都是信手拈来。** 另外，这本书的作者加多自身也会经常在网上发布各种技术文章。我觉得这本书也是加多大佬这么多年在多线程领域的沉淀所得的结果吧！他书中的内容基本都是结合代码讲解，非常有说服力！
- **[《实战 Java 高并发程序设计》](https://book.douban.com/subject/26663605/)**： 这个是我第二本要推荐的书籍，比较适合作为多线程入门/进阶书籍来看。这本书内容同样是理论结合实战，对于每个知识点的讲解也比较通俗易懂，整体结构也比较清。
- **[《深入浅出 Java 多线程》](https://github.com/RedSpider1/concurrent)**：这本书是几位大厂（如阿里）的大佬开源的，Github 地址：[https://github.com/RedSpider1/concurrent](https://github.com/RedSpider1/concurrent)几位作者为了写好《深入浅出 Java 多线程》这本书阅读了大量的 Java 多线程方面的书籍和博客，然后再加上他们的经验总结、Demo 实例、源码解析，最终才形成了这本书。这本书的质量也是非常过硬！给作者们点个赞！这本书有统一的排版规则和语言风格、清晰的表达方式和逻辑。并且每篇文章初稿写完后，作者们就会互相审校，合并到主分支时所有成员会再次审校，最后再通篇修订了三遍。
- **《Java 并发编程的艺术》** ：这本书不是很适合作为 Java 多线程入门书籍，需要具备一定的 JVM 基础，有些东西讲的还是挺深入的。另外，就我自己阅读这本书的感觉来说，我觉得这本书的章节规划有点杂乱，但是，具体到某个知识点又很棒！这可能也和这本书由三名作者共同编写完成有关系吧！
- ......

### JVM

-  **[《深入理解 Java 虚拟机（第 3 版）》](https://book.douban.com/subject/24722612/)）**：必读！必读！必读！神书，建议多刷几篇。里面不光有丰富地JVM理论知识，还有JVM实战案例！必读！
- **[《实战 JAVA 虚拟机》](https://book.douban.com/subject/26354292/)**：作为入门的了解 Java 虚拟机的知识还是不错的。

### Java8 新特性

- **[《Java 8 实战》](https://book.douban.com/subject/26772632/)**：面向 Java 8 的技能升级，包括 Lambdas、流和函数式编程特性。实战系列的一贯风格让自己快速上手应用起来。Java 8 支持的 Lambda 是精简表达在语法上提供的支持。Java 8 提供了 Stream，学习和使用可以建立流式编程的认知。
- **[《Java 8 编程参考官方教程》](https://book.douban.com/subject/26556574/)**：建议当做工具书来用！哪里不会翻哪里！

### 代码优化

-  **[《重构_改善既有代码的设计》](https://book.douban.com/subject/4262627/)**：豆瓣 9.1 分，重构书籍的开山鼻祖。
-  **[《Effective java 》](https://book.douban.com/subject/3360807/)**：本书介绍了在 Java 编程中很多极具实用价值的经验规则，这些经验规则涵盖了大多数开发人员每天所面临的问题的解决方案。这篇文章能够非常实际地帮助你写出更加清晰、健壮和高效的代码。本书中的每条规则都以简短、独立的小文章形式出现，并通过例子代码加以进一步说明。
-  **[《代码整洁之道》](https://book.douban.com/subject/5442024/)**：虽然是用 Java 语言作为例子，全篇都是在阐述 Java 面向对象的思想，但是其中大部分内容其它语言也能应用到。
-  **阿里巴巴 Java 开发手册** ：[https://github.com/alibaba/p3c](https://github.com/alibaba/p3c)
-  **Google Java 编程风格指南：** <http://www.hawstein.com/posts/google-java-style.html>

### 面试

1.  **[《JavaGuide面试突击版》](https://mp.weixin.qq.com/s?__biz=Mzg2OTA0Njk0OA==&mid=2247486324&idx=1&sn=e8b690ddaedabc486bd399310105aad3&chksm=cea244bff9d5cda9a627fa65235be09e7b089e92cf49c0eb0ceb35b39bbed86c1fab0125f5af&token=1745528586&lang=zh_CN&scene=21#wechat_redirect)** ：我的75k+ star的开源项目 [JavaGuide ](https://github.com/Snailclimb/JavaGuide) 转为面试浓缩而成的版本，不光提供了PDF版本（我的公众号JavaGuide后台回复：“面试突击”即可获取），在线阅读版本：[https://snailclimb.gitee.io/javaguide-interview/](https://snailclimb.gitee.io/javaguide-interview/)。
2. **[《Offer来了：Java面试核心知识点精讲》](https://book.douban.com/subject/34872163/)** : 这本书基本概括了Java程序员面试必备知识点，可以拿来准备Java面试或者夯实基础。不过，我还是更推荐我的 [JavaGuide](https://github.com/Snailclimb/JavaGuide) 和 **[《JavaGuide面试突击版》](https://mp.weixin.qq.com/s?__biz=Mzg2OTA0Njk0OA==&mid=2247486324&idx=1&sn=e8b690ddaedabc486bd399310105aad3&chksm=cea244bff9d5cda9a627fa65235be09e7b089e92cf49c0eb0ceb35b39bbed86c1fab0125f5af&token=1745528586&lang=zh_CN&scene=21#wechat_redirect)**  ，两者配合起来学习，真香！

## 网络

- **[《图解 HTTP》](https://book.douban.com/subject/25863515/)**： 讲漫画一样的讲 HTTP，很有意思，不会觉得枯燥，大概也涵盖也 HTTP 常见的知识点。因为篇幅问题，内容可能不太全面。不过，如果不是专门做网络方向研究的小伙伴想研究 HTTP 相关知识的话，读这本书的话应该来说就差不多了。
- **[《HTTP 权威指南》](https://book.douban.com/subject/10746113/)**:如果要全面了解 HTTP 非此书不可！

## 操作系统

- **[《鸟哥的 Linux 私房菜》](https://book.douban.com/subject/4889838/)**：本书是最具知名度的 Linux 入门书《鸟哥的 Linux 私房菜基础学习篇》的最新版，全面而详细地介绍了 Linux 操作系统。

## 数据结构

- **[《大话数据结构》](https://book.douban.com/subject/6424904/)**：入门类型的书籍，读起来比较浅显易懂，适合没有数据结构基础或者说数据结构没学好的小伙伴用来入门数据结构。

## 算法

### 入门

- **[《我的第一本算法书》](https://book.douban.com/subject/30357170/) （豆瓣评分 7.1，0.2K+人评价）** 一本不那么“专业”的算法书籍。和下面两本推荐的算法书籍都是比较通俗易懂，“不那么深入”的算法书籍。我个人非常推荐，配图和讲解都非常不错！
- **[《算法图解》](https://book.douban.com/subject/26979890/)（豆瓣评分 8.4，1.5K+人评价）** ：入门类型的书籍，读起来比较浅显易懂，非常适合没有算法基础或者说算法没学好的小伙伴用来入门。示例丰富，图文并茂，以让人容易理解的方式阐释了算法.读起来比较快，内容不枯燥！
- **[《啊哈!算法》](https://book.douban.com/subject/25894685/) （豆瓣评分 7.7，0.5K+人评价）** ：和《算法图解》类似的算法趣味入门书籍。

### 经典

> 下面这些书籍都是经典中的经典，但是阅读起来难度也比较大，不做太多阐述，神书就完事了！推荐先看 《算法》，然后再选下面的书籍进行进一步阅读。不需要都看，找一本好好看或者找某本书的某一个章节知识点好好看。

- **[《算法 第四版》](https://book.douban.com/subject/10432347/)（豆瓣评分 9.3，0.4K+人评价）：** 我在大二的时候被我们的一个老师强烈安利过！自己也在当时购买了一本放在宿舍，到离开大学的时候自己大概看了一半多一点。因为内容实在太多了！另外，这本书还提供了详细的Java代码，非常适合学习 Java 的朋友来看，可以说是 Java 程序员的必备书籍之一了。再来介绍一下这本书籍吧！这本书籍算的上是算法领域经典的参考书，全面介绍了关于算法和数据结构的必备知识，并特别针对排序、搜索、图处理和字符串处理进行了论述。
- **[编程珠玑](https://book.douban.com/subject/3227098/)（豆瓣评分 9.1，2K+人评价）** ：经典名著，被无数读者强烈推荐的书籍，几乎是顶级程序员必看的书籍之一了。这本书的作者也非常厉害，Java之父 James Gosling 就是他的学生。很多人都说这本书不是教你具体的算法，而是教你一种编程的思考方式。这种思考方式不仅仅在编程领域适用，在其他同样适用。
- **[《算法设计手册》](https://book.douban.com/subject/4048566/)（豆瓣评分9.1 ， 45人评价）** :被 [Teach Yourself Computer Science](https://teachyourselfcs.com/) 强烈推荐的一本算法书籍。
- **[《算法导论》](https://book.douban.com/subject/20432061/) （豆瓣评分 9.2，0.4K+人评价）**
- **[《计算机程序设计艺术（第1卷）》](https://book.douban.com/subject/1130500/)（豆瓣评分 9.4，0.4K+人评价）** 

### 面试

1. **[《剑指Offer》](https://book.douban.com/subject/6966465/)（豆瓣评分 8.3，0.7K+人评价）**这本面试宝典上面涵盖了很多经典的算法面试题，如果你要准备大厂面试的话一定不要错过这本书。《剑指Offer》 对应的算法编程题部分的开源项目解析：[CodingInterviews](https://github.com/gatieme/CodingInterviews)
2. **[程序员代码面试指南：IT名企算法与数据结构题目最优解（第2版）](https://book.douban.com/subject/30422021/) （豆瓣评分 8.7，0.2K+人评价）** ：题目相比于《剑指 offer》 来说要难很多，题目涵盖面相比于《剑指 offer》也更加全面。全书一共有将近300道真实出现过的经典代码面试题。
3. **[编程之美](https://book.douban.com/subject/3004255/)（豆瓣评分 8.4，3K+人评价）**：这本书收集了约60道算法和程序设计题目，这些题目大部分在近年的笔试、面试中出现过，或者是被微软员工热烈讨论过。作者试图从书中各种有趣的问题出发，引导读者发现问题，分析问题，解决问题，寻找更优的解法。

## 数据库

**MySQL:**

-  **[《高性能 MySQL》](https://book.douban.com/subject/23008813/)**：这本书不用多说了把！MySQL 领域的经典之作，拥有广泛的影响力。不但适合数据库管理员（dba）阅读，也适合开发人员参考学习。不管是数据库新手还是专家，相信都能从本书有所收获。如果你的时间不够的话，第5章关于索引的内容和第6章关于查询的内容是必读的！
-  [《MySQL 技术内幕-InnoDB 存储引擎》](<https://book.douban.com/subject/24708143/>)（推荐，豆瓣评分 8.7）：了解 InnoDB 存储引擎底层原理必备的一本书，比较深入。

**Redis:**

-  **[《Redis 实战》](https://book.douban.com/subject/26612779/)**：如果你想了解 Redis 的一些概念性知识的话，这本书真的非常不错。
-  **[《Redis 设计与实现》](https://book.douban.com/subject/25900156/)**：也还行吧！

## 系统设计

### 设计模式

- **[《设计模式 : 可复用面向对象软件的基础》](https://book.douban.com/subject/1052241/)** ：设计模式的经典！
- **[《Head First 设计模式（中文版）》](https://book.douban.com/subject/2243615/)** ：相当赞的一本设计模式入门书籍。用实际的编程案例讲解算法设计中会遇到的各种问题和需求变更（对的，连需求变更都考虑到了！），并以此逐步推导出良好的设计模式解决办法。
- **[《大话设计模式》](https://book.douban.com/subject/2334288/)** ：本书通篇都是以情景对话的形式，用多个小故事或编程示例来组织讲解GOF(即《设计模式 : 可复用面向对象软件的基础》这本书))，但是不像《设计模式 : 可复用面向对象软件的基础》难懂。但是设计模式只看书是不够的，还是需要在实际项目中运用，在实战中体会。

### 常用框架

#### Spring/SpringBoot

- **[《Spring 实战（第 4 版）》](https://book.douban.com/subject/26767354/)** ：不建议当做入门书籍读，入门的话可以找点国人的书或者视频看。这本定位就相当于是关于 Spring 的新华字典，只有一些基本概念的介绍和示例，涵盖了 Spring 的各个方面，但都不够深入。就像作者在最后一页写的那样：“学习 Spring，这才刚刚开始”。
- **《[Spring源码深度解析 第2版](https://book.douban.com/subject/30452948/)》**  ：读Spring源码必备的一本书籍。市面上关于Spring源码分析的书籍太少了。
- **[《Spring 5高级编程（第5版）》](https://book.douban.com/subject/30452637/)** ：推荐阅读，对于Spring5的新特性介绍的很好！不过内容比较多，可以作为工具书参考。
-  **[《精通Spring4.x企业应用开发实战》](https://read.douban.com/ebook/58113975/?dcs=subject-rec&dcm=douban&dct=26767354)** :通过实战讲解，比较适合作为Spring入门书籍来看。
- **[《Spring入门经典》](https://book.douban.com/subject/26652876/)** ：适合入门，也有很多示例！
-  **[《Spring Boot实战派》](https://book.douban.com/subject/34894533/)** ：这本书使用的Spring Boot 2.0+的版本，还算比较新。整本书采用“知识点+实例”的形式编写。本书通过“58个基于知识的实例+2个综合性的项目”，深入地讲解Spring Boot的技术原理、知识点和具体应用；把晦涩难懂的理论用实例展现出来，使得读者对知识的理解变得非常容易，同时也立即学会如何使用它。说实话，我还是比较推荐这本书的。
- **[《Spring Boot编程思想（核心篇）》](https://book.douban.com/subject/33390560/)** ：SpringBoot深入书，不适合初学者。书尤其的厚，这本书的缺点是书的很多知识点的讲解过于啰嗦和拖沓，优点是书中对SpringBoot内部原理讲解很清楚。

#### Netty

- **[《Netty进阶之路：跟着案例学Netty》](https://book.douban.com/subject/30381214/)** : 这本书的优点是有不少实际的案例的讲解，通过案例来学习是很不错的！
- **[《Netty 4.x 用户指南》](https://waylau.gitbooks.io/netty-4-user-guide/content/)** ：《Netty 4.x 用户指南》中文翻译（包含了官方文档以及其他文章）。
- **[《Netty 入门与实战：仿写微信 IM 即时通讯系统》](https://juejin.im/book/5b4bc28bf265da0f60130116?referrer=59fbb2daf265da4319559f3a)**   ：基于 Netty 框架实现 IM 核心系统，带你深入学习 Netty 网络编程核心知识
- **[《Netty 实战》](https://book.douban.com/subject/27038538/)** ：可以作为工具书参考！

### 分布式

- **[《从 Paxos 到 Zookeeper》](https://book.douban.com/subject/26292004/)**：简要介绍几种典型的分布式一致性协议，以及解决分布式一致性问题的思路，其中重点讲解了 Paxos 和 ZAB 协议。同时，本书深入介绍了分布式一致性问题的工业解决方案——ZooKeeper，并着重向读者展示这一分布式协调框架的使用方法、内部实现及运维技巧，旨在帮助读者全面了解 ZooKeeper，并更好地使用和运维 ZooKeeper。
- **[《RabbitMQ 实战指南》](https://book.douban.com/subject/27591386/)**：《RabbitMQ 实战指南》从消息中间件的概念和 RabbitMQ 的历史切入，主要阐述 RabbitMQ 的安装、使用、配置、管理、运维、原理、扩展等方面的细节。如果你想浅尝 RabbitMQ 的使用，这本书是你最好的选择；如果你想深入 RabbitMQ 的原理，这本书也是你最好的选择；总之，如果你想玩转 RabbitMQ，这本书一定是最值得看的书之一
- **[《Spring Cloud 微服务实战》](https://book.douban.com/subject/27025912/)**：从时下流行的微服务架构概念出发，详细介绍了 Spring Cloud 针对微服务架构中几大核心要素的解决方案和基础组件。对于各个组件的介绍，《Spring Cloud 微服务实战》主要以示例与源码结合的方式来帮助读者更好地理解这些组件的使用方法以及运行原理。同时，在介绍的过程中，还包含了作者在实践中所遇到的一些问题和解决思路，可供读者在实践中作为参考。

### 网站架构

-  **[《大型网站技术架构：核心原理与案例分析+李智慧》](https://book.douban.com/subject/25723064/)**:这本书我读过，基本不需要你有什么基础啊~读起来特别轻松，但是却可以学到很多东西，非常推荐了。另外我写过这本书的思维导图，关注我的微信公众号：“Java 面试通关手册”回复“大型网站技术架构”即可领取思维导图。
- **[《亿级流量网站架构核心技术》](https://book.douban.com/subject/26999243/)**：一书总结并梳理了亿级流量网站高可用和高并发原则，通过实例详细介绍了如何落地这些原则。本书分为四部分：概述、高可用原则、高并发原则、案例实战。从负载均衡、限流、降级、隔离、超时与重试、回滚机制、压测与预案、缓存、池化、异步化、扩容、队列等多方面详细介绍了亿级流量网站的架构核心技术，让读者看后能快速运用到实践项目中。
- **[《从零开始学架构（李运华）》](https://book.douban.com/subject/30335935/)** ： 这本书对应的有一个极客时间的专栏—《从零开始学架构》，里面的很多内容都是这个专栏里面的，两者买其一就可以了。我看了很小一部分，内容挺全面的，是一本真正在讲如何做架构的书籍。
- **[《架构修炼之道——亿级网关、平台开放、分布式、微服务、容错等核心技术修炼实践》](https://book.douban.com/subject/33389549/)** ：非常喜欢的一本书，对一些知识点比如消息队列、API网管讲解的很好,通俗易懂。

### 底层

- **[《深入剖析 Tomcat》](https://book.douban.com/subject/10426640/)**：本书深入剖析 Tomcat 4 和 Tomcat 5 中的每个组件，并揭示其内部工作原理。通过学习本书，你将可以自行开发 Tomcat 组件，或者扩展已有的组件。 读完这本书，基本可以摆脱背诵面试题的尴尬。
- **[《深入理解 Nginx（第 2 版）》](https://book.douban.com/subject/26745255/)**：作者讲的非常细致，注释都写的都很工整，对于 Nginx 的开发人员非常有帮助。优点是细致，缺点是过于细致，到处都是代码片段，缺少一些抽象。

## 软件设计之道

- **[《人月神话》](https://book.douban.com/subject/1102259/)** ：  非常值得阅读的一本书籍。看书名感觉的第一眼感觉不像是技术类的书籍。这本书对于现代软件尤其是复杂软件的开发的规范化有深刻的意义。
- **《领域驱动设计:软件核心复杂性应对之道》** ： 这本领域驱动设计方面的经典之作一直被各种推荐，但是我还来及读。

## 其他

- **[《黑客与画家》](https://read.douban.com/ebook/387525/?dcs=subject-rec&dcm=douban&dct=2243615)**：这本书是硅谷创业之父，Y Combinator 创始人 Paul Graham 的文集。之所以叫这个名字，是因为作者认为黑客（并非负面的那个意思）与画家有着极大的相似性，他们都是在创造，而不是完成某个任务。

- **[《图解密码技术》](https://book.douban.com/subject/26265544/)**：本书以**图配文**的形式，第一部分讲述了密码技术的历史沿革、对称密码、分组密码模式（包括ECB、CBC、CFB、OFB、CTR）、公钥、混合密码系统。第二部分重点介绍了认证方面的内容，涉及单向散列函数、消息认证码、数字签名、证书等。第三部分讲述了密钥、随机数、PGP、SSL/TLS 以及密码技术在现实生活中的应用。关键字：JWT 前置知识、区块链密码技术前置知识。属于密码知识入门书籍。、
- 《程序开发心理学》 、《程序员修炼之道，从小工道专家》、 《高效程序员的45个习惯，敏捷开发修炼之道》 、《高效能程序员的修炼》 、《软技能，代码之外的生存之道》  、《程序员的职业素养》 、《程序员的思维修炼》

  




